"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wrapRange = exports.removeElemsWrapping = exports.isSideLabel = exports.getRangeDetails = exports.getLabelElement = exports.getDOMNodes = exports.getAnnotationElements = exports.clearSelection = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

// returns DOM Nodes before the stopOffset
var getAllTextNodesBeforePosition = function getAllTextNodesBeforePosition(node, stopOffset, nodeArray) {
  var nodes = nodeArray ? nodeArray : [];
  var offset = nodes.reduce(function (acc, node) {
    return acc + node.textContent.length;
  }, 0);
  var keepWalking = true;

  if (offset > stopOffset) {
    return false;
  }

  if (node) {
    // 3 = HTML DOM node value for text nodes
    if (node.nodeType === 3 && node.parentNode && !node.parentNode.hasAttribute('data-ann-id')) {
      nodes.push(node);
    }

    node = node.firstChild;
  }

  while (node && keepWalking) {
    keepWalking = getAllTextNodesBeforePosition(node, stopOffset, nodes);
    node = node.nextSibling;
  }

  return nodes;
}; // returns DOM Nodes that overlap the start or end position


var getDOMNodes = function getDOMNodes(startOffset, endOffset, rootNode) {
  var textNodesToEnd = getAllTextNodesBeforePosition(rootNode, endOffset);
  var domNodes = [];
  var start = 0;
  textNodesToEnd.forEach(function (node) {
    var end = start + node.textContent.length;
    [startOffset, endOffset].forEach(function (offset) {
      if (start <= offset && offset < end) {
        domNodes.push({
          node: node,
          offset: offset - start
        });
      }
    });
    start = end;
  });
  return domNodes;
}; // returns [ textNodes, text ]
// textNodes = DOM Text Nodes between start range and end range node
// text = plain text in range


exports.getDOMNodes = getDOMNodes;

var getTextNodesBetween = function getTextNodesBetween(range) {
  var rootNode = range.commonAncestorContainer,
      startNode = range.startContainer,
      endNode = range.endContainer,
      startOffset = range.startOffset,
      endOffset = range.endOffset;
  var textNodes = [];
  var pastStartNode = false;
  var reachedEndNode = false;
  var text = '';

  var getTextNodes = function getTextNodes(node) {
    var value = node.nodeValue,
        childNodes = node.childNodes;

    if (node === startNode && node === endNode) {
      if (value) {
        text += value.substring(startOffset, endOffset);
      }

      pastStartNode = reachedEndNode = true;
    } else if (node === startNode) {
      if (value) {
        text += value.substring(startOffset);
      }

      pastStartNode = true;
    } else if (node === endNode) {
      if (value) {
        text += value.substring(0, endOffset);
      }

      reachedEndNode = true;
    } else if (node && node.nodeType === 3 && node.parentNode && !node.parentNode.hasAttribute('data-ann-id')) {
      // 3 = HTML DOM node value for text nodes
      if (value && pastStartNode && !reachedEndNode) {
        text += value;
        textNodes.push(node);
      }
    }

    childNodes.forEach(function (childNode) {
      if (!reachedEndNode) {
        getTextNodes(childNode);
      } else {
        return;
      }
    });
  };

  getTextNodes(rootNode);
  return [textNodes, text];
};

var surroundContent = function surroundContent(range) {
  var wrapper = document.createElement('SPAN');
  range.surroundContents(wrapper);
  return wrapper;
}; // wrap each DOM Text Node in range into a span


var wrapRange = function wrapRange(range) {
  if (range.startContainer === range.endContainer) {
    return [surroundContent(range)];
  }

  var _getTextNodesBetween = getTextNodesBetween(range),
      _getTextNodesBetween2 = (0, _slicedToArray2["default"])(_getTextNodesBetween, 1),
      nodesBetween = _getTextNodesBetween2[0]; // wrap the start node


  var startRange = document.createRange();
  startRange.selectNodeContents(range.startContainer);
  startRange.setStart(range.startContainer, range.startOffset);
  var startWrapper = surroundContent(startRange); // wrap the end node

  var endRange = document.createRange();
  endRange.selectNode(range.endContainer);
  endRange.setEnd(range.endContainer, range.endOffset);
  var endWrapper = surroundContent(endRange); // wrap the nodes between start and end nodes, if any

  var centerWrappers = nodesBetween.map(function (node) {
    var wrapper = document.createElement('SPAN');
    node.parentNode.insertBefore(wrapper, node);
    wrapper.appendChild(node);
    return wrapper;
  });
  return [startWrapper].concat((0, _toConsumableArray2["default"])(centerWrappers), [endWrapper]);
}; // returns text in range with start and end position in rootNode


exports.wrapRange = wrapRange;

var getRangeDetails = function getRangeDetails(range, rootNode) {
  var rangeBefore = document.createRange();
  rangeBefore.setStart(rootNode, 0);
  rangeBefore.setEnd(range.startContainer, range.startOffset);

  var _getTextNodesBetween3 = getTextNodesBetween(rangeBefore),
      _getTextNodesBetween4 = (0, _slicedToArray2["default"])(_getTextNodesBetween3, 2),
      text = _getTextNodesBetween4[1];

  var _getTextNodesBetween5 = getTextNodesBetween(range),
      _getTextNodesBetween6 = (0, _slicedToArray2["default"])(_getTextNodesBetween5, 2),
      textSelected = _getTextNodesBetween6[1];

  var start = text.length;
  var end = start + textSelected.length;
  return {
    quote: textSelected,
    start: start,
    end: end
  };
}; // removes the annotation spans without changing the content inside


exports.getRangeDetails = getRangeDetails;

var removeElemsWrapping = function removeElemsWrapping(elems, container) {
  (elems || []).forEach(function (elem) {
    var parent = elem.parentNode;
    var childNodes = elem.childNodes;
    var childNodesLength = childNodes.length;

    if (childNodesLength > 0) {
      for (var i = 0; i < childNodesLength; i++) {
        parent.insertBefore(childNodes[0], elem);
      }
    } else {
      parent.insertBefore(document.createTextNode(elem.textContent), elem);
    }

    parent.removeChild(elem);
  });
  container.normalize();
}; // also used in select-text


exports.removeElemsWrapping = removeElemsWrapping;

var clearSelection = function clearSelection() {
  if (document.getSelection) {
    // for all new browsers (IE9+, Chrome, Firefox)
    document.getSelection().removeAllRanges();
    document.getSelection().addRange(document.createRange());
  } else if (window.getSelection) {
    // equals with the document.getSelection (MSDN info)
    if (window.getSelection().removeAllRanges) {
      // for all new browsers (IE9+, Chrome, Firefox)
      window.getSelection().removeAllRanges();
      window.getSelection().addRange(document.createRange());
    } else if (window.getSelection().empty) {
      // Chrome supports this as well
      window.getSelection().empty();
    }
  } else if (document.selection) {
    // IE8-
    document.selection.empty();
  }
}; // check if annotation label contains line breaks or its length >= 20 characters


exports.clearSelection = clearSelection;

var isSideLabel = function isSideLabel(text) {
  return text.length >= 20 || text.search(/\n|\r|\r\n/) !== -1;
};

exports.isSideLabel = isSideLabel;

var getAnnotationElements = function getAnnotationElements(id) {
  return Array.from(document.querySelectorAll("[data-id='".concat(id, "']")));
};

exports.getAnnotationElements = getAnnotationElements;

var getLabelElement = function getLabelElement(id) {
  return document.querySelector("[data-ann-id='".concat(id, "']"));
};

exports.getLabelElement = getLabelElement;
//# sourceMappingURL=annotation-utils.js.map